@@ -14,7 +14,10 @@
   .pkgenv$threads = 1
   .pkgenv$debug = FALSE
   .pkgenv$ncdf_write_bounds = TRUE 
+  .pkgenv$use_overview_images = TRUE
+
   #.pkgenv$swarm = NULL
+  register_s3_method("stars","st_as_stars", "cube")
   
   # for windows, rwinlib includes GDAL data and PROJ data in the package and we must set the environment variables
   # PROJ_LIB and GDAL_DATA to make sure GDAL finds the data at the package location
@@ -45,8 +48,35 @@
   if(Sys.getenv("GDALCUBES_STREAMING") == "1") {
     sink(stderr())
   }
-  else {
-    x = libgdalcubes_version()
-    packageStartupMessage(paste("Using gdalcubes library version ", x$VERSION_MAJOR, ".", x$VERSION_MINOR, ".", x$VERSION_PATCH, sep=""))
-  }
+  #else {
+  #  x = libgdalcubes_version()
+  #  #packageStartupMessage(paste("Using gdalcubes library version ", x$VERSION_MAJOR, ".", x$VERSION_MINOR, ".", x$VERSION_PATCH, sep=""))
+  #}
 }
+
+
+# From https://github.com/tidyverse/hms/blob/a24d877fb439486f9e3fb5fab35e178aecaa858d/R/zzz.R (accessed 2020-06-03)
+register_s3_method <- function(pkg, generic, class, fun = NULL) {
+  stopifnot(is.character(pkg), length(pkg) == 1)
+  stopifnot(is.character(generic), length(generic) == 1)
+  stopifnot(is.character(class), length(class) == 1)
+  
+  if (is.null(fun)) {
+    fun <- get(paste0(generic, ".", class), envir = parent.frame())
+  } else {
+    stopifnot(is.function(fun))
+  }
+  
+  if (pkg %in% loadedNamespaces()) {
+    registerS3method(generic, class, fun, envir = asNamespace(pkg))
+  }
+  
+  # Always register hook in case package is later unloaded & reloaded
+  setHook(
+    packageEvent(pkg, "onLoad"),
+    function(...) {
+      registerS3method(generic, class, fun, envir = asNamespace(pkg))
+    }
+  )
+}
+
