@@ -23,17 +23,20 @@
 */
 
 #include "view.h"
+
+#include <fstream>
+
 #include "filesystem.h"
 
 namespace gdalcubes {
 
-cube_view cube_view::read(nlohmann::json j) {
+cube_view cube_view::read(json11::Json j) {
     cube_view v;
 
-    v._dt = duration::from_string(j.at("time").at("dt").get<std::string>());
+    v._dt = duration::from_string(j["time"]["dt"].string_value());
 
-    std::string st0 = j.at("time").at("t0").get<std::string>();
-    std::string st1 = j.at("time").at("t1").get<std::string>();
+    std::string st0 = j["time"]["t0"].string_value();
+    std::string st1 = j["time"]["t1"].string_value();
 
     v._t0 = datetime::from_string(st0);
     v._t1 = datetime::from_string(st1);
@@ -42,30 +45,30 @@ cube_view cube_view::read(nlohmann::json j) {
     v._t0.unit() = v._dt.dt_unit;
     v._t1.unit() = v._dt.dt_unit;
 
-    if (j.count("space")) {
-        v._win.left = j.at("space").at("left").get<double>();
-        v._win.right = j.at("space").at("right").get<double>();
-        v._win.top = j.at("space").at("top").get<double>();
-        v._win.bottom = j.at("space").at("bottom").get<double>();
+    if (!j["space"].is_null()) {
+        v._win.left = j["space"]["left"].number_value();
+        v._win.right = j["space"]["right"].number_value();
+        v._win.top = j["space"]["top"].number_value();
+        v._win.bottom = j["space"]["bottom"].number_value();
 
-        if (j.at("space").count("nx") && (j.at("space").count("ny"))) {
-            v._nx = j.at("space").at("nx").get<uint32_t>();
-            v._ny = j.at("space").at("ny").get<uint32_t>();
+        if (!j["space"]["nx"].is_null() && !j["space"]["ny"].is_null()) {
+            v._nx = j["space"]["nx"].int_value();
+            v._ny = j["space"]["ny"].int_value();
         }
         // if only one of nx and ny is given, use the aspect ratio of the spatial extent to derive the other
-        else if (j.at("space").count("nx") && !(j.at("space").count("ny"))) {
-            v._nx = j.at("space").at("nx").get<uint32_t>();
+        else if (!j["space"]["nx"].is_null() && j["space"]["ny"].is_null()) {
+            v._nx = j["space"]["nx"].int_value();
             v._ny = (uint32_t)((double)v._nx * (v._win.top - v._win.bottom) / (v._win.right - v._win.left));
-        } else if (!j.at("space").count("nx") && (j.at("space").count("ny"))) {
-            v._ny = j.at("space").at("ny").get<uint32_t>();
+        } else if (j["space"]["nx"].is_null() && !j["space"]["ny"].is_null()) {
+            v._ny = j["space"]["ny"].int_value();
             v._nx = (uint32_t)((double)v._ny * (v._win.right - v._win.left) / (v._win.top - v._win.bottom));
         } else {
             throw std::string("ERROR in cube_view::read_json_string(): at least one of nx or ny must be given.");
         }
 
-        v._srs = j.at("space").at("srs").get<std::string>();
+        v._srs = j["space"]["srs"].string_value();
 
-    } else if (j.count("tile")) {
+    } else if (!j["tile"].is_null()) {
         const double EARTH_RADIUS_METERS = 6378137;
         const uint16_t TILE_SIZE_PX = 256;
         const double EARTH_CIRCUMFERENCE_METERS = 2 * M_PI * EARTH_RADIUS_METERS;
@@ -74,9 +77,9 @@ cube_view cube_view::read(nlohmann::json j) {
         //const double WEBMERCATOR_BOUNDS_RIGHT = EARTH_CIRCUMFERENCE_METERS / 2.0;   // 20037508.342789244
         const double WEBMERCATOR_BOUNDS_UPPER = EARTH_CIRCUMFERENCE_METERS / 2.0;  // 20037508.342789244
 
-        uint32_t x = j.at("tile").at("x").get<uint32_t>();
-        uint32_t y = j.at("tile").at("y").get<uint32_t>();
-        uint32_t z = j.at("tile").at("z").get<uint32_t>();
+        uint32_t x = j["tile"]["x"].int_value();
+        uint32_t y = j["tile"]["y"].int_value();
+        uint32_t z = j["tile"]["z"].int_value();
 
         bounds_2d<double> win;
         win.left = WEBMERCATOR_BOUNDS_LEFT + x * EARTH_CIRCUMFERENCE_METERS / std::pow(2, z);
@@ -93,16 +96,16 @@ cube_view cube_view::read(nlohmann::json j) {
         throw std::string("ERROR in cube_view::read(): expected either 'space' or 'tile' in JSON cube view");
     }
 
-    if (j.count("resampling") == 0) {
+    if (j["resampling"].is_null()) {
         v._resampling = resampling::resampling_type::RSMPL_NEAR;
     } else {
-        v._resampling = resampling::from_string(j.at("resampling").get<std::string>());
+        v._resampling = resampling::from_string(j["resampling"].string_value());
     }
 
-    if (j.count("aggregation") == 0) {
+    if (j["aggregation"].is_null()) {
         v._aggregation = aggregation::aggregation_type::AGG_NONE;
     } else {
-        v._aggregation = aggregation::from_string(j.at("aggregation").get<std::string>());
+        v._aggregation = aggregation::from_string(j["aggregation"].string_value());
     }
 
     return v;
@@ -112,24 +115,26 @@ cube_view cube_view::read_json(std::string filename) {
     if (!filesystem::exists(filename))
         throw std::string("ERROR in cube_view::read_json(): image_collection_cube view file does not exist.");
     std::ifstream i(filename);
-    nlohmann::json j;
-    i >> j;
+
+    std::stringstream buf;
+    buf << i.rdbuf();
+
+    std::string err;  // TODO: do something with error
+    json11::Json j = json11::Json::parse(buf.str(), err);
     return read(j);
 }
 
 cube_view cube_view::read_json_string(std::string str) {
     std::istringstream i(str);
-    nlohmann::json j;
-    i >> j;
-    cube_view v;
-
+    std::string err;  // TODO: do something with error
+    json11::Json j = json11::Json::parse(str, err);
     return read(j);
 }
 
 void cube_view::write_json(std::string filename) {
-    nlohmann::json j = nlohmann::json{
-        {"space", {{"nx", _nx}, {"ny", _ny}, {"left", _win.left}, {"right", _win.right}, {"top", _win.top}, {"bottom", _win.bottom}, {"srs", _srs}}},
-        {"time", {{"dt", dt().to_string()}, {"t0", _t0.to_string()}, {"t1", _t1.to_string()}}},
+    json11::Json j = json11::Json::object{
+        {"space", json11::Json::object{{"nx", (int)_nx}, {"ny", (int)_ny}, {"left", _win.left}, {"right", _win.right}, {"top", _win.top}, {"bottom", _win.bottom}, {"srs", _srs}}},
+        {"time", json11::Json::object{{"dt", dt().to_string()}, {"t0", _t0.to_string()}, {"t1", _t1.to_string()}}},
         {"aggregation", aggregation::to_string(_aggregation)},
         {"resampling", resampling::to_string(_resampling)}};
 
@@ -138,19 +143,18 @@ void cube_view::write_json(std::string filename) {
         throw std::string("ERROR in cube_view::write_json(): cannot write to file.");
     }
 
-    o << std::setw(4) << j << std::endl;
+    o << std::setw(4) << j.dump() << std::endl;
     o.close();
 }
 
 std::string cube_view::write_json_string() {
-    nlohmann::json j = nlohmann::json{
-        {"space", {{"nx", _nx}, {"ny", _ny}, {"left", _win.left}, {"right", _win.right}, {"top", _win.top}, {"bottom", _win.bottom}, {"srs", _srs}}},
-        {"time", {{"dt", dt().to_string()}, {"t0", _t0.to_string()}, {"t1", _t1.to_string()}}},
+    json11::Json j = json11::Json::object{
+        {"space", json11::Json::object{{"nx", (int)_nx}, {"ny", (int)_ny}, {"left", _win.left}, {"right", _win.right}, {"top", _win.top}, {"bottom", _win.bottom}, {"srs", _srs}}},
+        {"time", json11::Json::object{{"dt", dt().to_string()}, {"t0", _t0.to_string()}, {"t1", _t1.to_string()}}},
         {"aggregation", aggregation::to_string(_aggregation)},
         {"resampling", resampling::to_string(_resampling)}};
     std::ostringstream o;
-    o << j;
-    return o.str();
+    return j.dump();
 }
 
 }  // namespace gdalcubes
